/* Cellular Automata with totalistic rules transition Y=F(sum(W*Y)), where W - weighted matrix, Y - cellular array */

/* -------------- Simulation parameters ------------------------------------------------------*/
int SizeX = 200, SizeY = 200 ; // cellular array size
int Block = 7 ; // templates size
double n=-1. ; // inhibitor n < 0
double p=1. ; // activator p > 0
double probab=0.005 ; // probability that y[i][j]=1 for initialization
int[][] points={{SizeX/2,SizeY/2}} ; // coordinates of 1 in cellular array for initialization
// initial state of array is the following: array is divided in parts with it's own probability of y[i][j]=1. Ones distributed with different probability in different parts. 
double p0=1E-4 ; // beginning of probability range
double p1=0.1 ; // end of probability range
int sections=3 ; // number of parts
// initial state of array is the following: ring with center (cx,cy) big radius R1 and small radius R2
int cx=SizeX/2 ;
int cy=SizeY/2 ;
int R1=SizeX/3 ;
int R2=SizeX/4 ;
/* ------------------------------------------------------------------------------------------*/ 

int[][] ys, ya ;
double[][] W ;
int[][][] template ; 
int bx;
int by;
int shift ;
boolean Start=false ;
float Scale=1. ;

void setup() 
{ 
  int i, j, k, l ;
  size(SizeX*2+30, SizeY*2+30) ; // size of window

  frameRate(25) ;
  ys = new int[SizeX][SizeY] ; 
  ya = new int[SizeX][SizeY] ; 
  template = new int[Block][Block][2] ;
  W = new double[Block][Block] ;
  
  // Set cells to '0' 
  for(i=0;i<SizeX;i++)
    for(j=0;j<SizeY;j++) 
      ys[i][j]=ya[i][j]=0 ;

 for(i=-Block/2,k=0;i<=Block/2;i++,k++)
   for(j=-Block/2,l=0;j<=Block/2;j++,l++)
   {
     template[k][l][0]=i ; 
     template[k][l][1]=j ;
   }

  for(i=0;i<Block;i++)
    for(j=0;j<Block;j++)
       W[i][j]=((i%(Block-1))==0 || (j%(Block-1))==0)? n:p ; 
 
  bx=0 ;
  by=0 ;
  translate(10,10);
  background(255);
  fill(255) ;
  stroke(0) ;
  rect(bx-1, by-1, SizeX+1, SizeY+1) ;
  shift=SizeX+10 ;
  rect(bx+shift-1, by-1, SizeX+1, SizeY+1) ;

  colorMode(HSB, 100);
}

void draw() 
{
  int i, j ;

  translate(10,10);  
  fill(255) ;
  stroke(0) ;
  rect(bx-1, by+shift-1, SizeX+1, SizeY+1) ;
  rect(bx+shift-1, by+shift-1, SizeX+1, SizeY+1) ;

  if(Start)
  {
    calc_sync() ;
    calc_async() ;
  }
  
  for(i=0;i<SizeX;i++)
    for(j=0;j<SizeY;j++)
    {
      if(ys[i][j]==1)
      {
        stroke(70, 100, 100);
        point(i,j+shift) ;
        if(!Start) point(i,j) ;
      }
      else 
      {
        stroke(0, 0, 100);
        point(i,j+shift) ;
        if(!Start) point(i,j) ;
      }
      if(ya[i][j]==1)
      {
        stroke(90, 100, 100);
        point(i+shift,j+shift) ;
        if(!Start) point(i+shift,j) ;
      }  
      else 
      {
        stroke(0, 0, 100);
        point(i+shift,j+shift) ;
        if(!Start) point(i+shift,j) ;
      }
    }
}

void calc_sync()
{
  int i, j ;
  int[][] x=new int [SizeX][SizeY] ;
  
  for(i=0;i<SizeX;i++)
    for(j=0;j<SizeY;j++)
      x[i][j]=function(sum(i,j,W,ys)) ;

  for(i=0;i<SizeX;i++)
    for(j=0;j<SizeY;j++)
      ys[i][j]=x[i][j] ;
}

void calc_async()
{
  int i, j, k ;
  
  for(k=0;k<SizeX*SizeY;k++)
  {
    i=(int)random(SizeX) ;      
    j=(int)random(SizeY) ;
    ya[i][j]=function(sum(i,j,W,ya)) ;
   }
}


double sum(int x, int y, double[][] a, int[][] b)
{
  int i, j ;
  double sum=0. ;
  
  for(i=0;i<Block;i++)
    for(j=0;j<Block;j++)
      sum+=a[i][j]*b[(x+template[i][j][0]+SizeX)%SizeX][(y+template[i][j][1]+SizeY)%SizeY] ;

  return sum ;      
}

int function(double var)
{
  return (var<=1E-15)? 0:1 ;  
}

void mouseReleased()
{
 if(!Start)
 {
    // Test if the cursor is over the box 
    if (mouseX > bx && mouseX < bx+SizeX && mouseY > by && mouseY < by+SizeY) 
    {
        stroke(70, 100, 100);
        point(mouseX,mouseY) ;
        println("Synchronous CA: point(" + mouseX + " ; " + mouseY + ")") ;
        ys[mouseX][mouseY]=1 ;
    }
    else if (mouseX > bx+shift && mouseX < bx+shift+SizeX && mouseY > by && mouseY < by+SizeY) 
    {
        stroke(90,100,100);
        point(mouseX,mouseY) ;
        println("Asynchronous CA: point(" + (mouseX-shift) + " ;  " + mouseY + ")") ;
        ya[mouseX-shift][mouseY]=1 ;
    }
 }
}

void mouseDragged() 
{
  if(!Start)
  {
    if (mouseX > bx && mouseX < bx+SizeX && mouseY > by && mouseY < by+SizeY)
    {
      stroke(70, 100, 100);
      if( pmouseX > bx && pmouseX < bx+SizeX && pmouseY > by && pmouseY < by+SizeY) lineCA(pmouseX, mouseX, pmouseY, mouseY, ys, 0) ;
    }
    else if (mouseX > bx+shift && mouseX < bx+shift+SizeX && mouseY > by && mouseY < by+SizeY) 
    {
      stroke(90,100,100);
      if (pmouseX > bx+shift && pmouseX < bx+shift+SizeX && pmouseY > by && pmouseY < by+SizeY)  lineCA(pmouseX, mouseX, pmouseY, mouseY, ya, shift) ;
    }
  }  
}

void lineCA(int x0, int x1, int y0, int y1, int[][] arr, int sh)
{
  int tmp ;
  boolean slope=abs(y1 - y0) > abs(x1 - x0) ;
  
  if(slope) // if negative slope then rotation about x=y 
  {
    tmp=x0 ; // swap(x0, y0) ;
    x0=y0 ;
    y0=tmp ;
    tmp=x1 ; // swap(x1, y1) ;
    x1=y1 ;
    y1=tmp ;
  }
  if( x0 > x1) // if line go from right to left then exchange x0 and x
  {
    tmp=x0 ; // swap(x0, x1) ;
    x0=x1 ;
    x1=tmp ;
    tmp=y0 ; // swap(y0, y1) ;
    y0=y1 ;
    y1=tmp ;
  }
  int deltax=x1-x0 ;
  int deltay=abs(y1-y0) ;
  int error=deltax/2 ;
  int ystep ;
  int y=y0, x ;
  
  if(y0 < y1) // if line go down 
    ystep=1 ; 
  else 
    ystep=-1 ; // if line go up 
    
  for(x=x0; x<=x1; x++)
  {
    if(slope)
    {
      arr[y-sh][x]=1 ; // so changed slope of line, write y in x and x in y
      point(y,x) ;
    }
    else 
    {
      arr[x-sh][y]=1 ;
      point(x,y) ;
    }  
    error=error-deltay ;
    if(error < 0)
    {
      y+=ystep ;
      error+=deltax ;
    }
  }  
}

void keyPressed()
{
  int i, j ;
  
  if(key=='0')
  {
    background(0,0,100);
    fill(0,0,100) ;
    stroke(0,-100,-100) ;
    rect(bx-1, by-1, SizeX+1, SizeY+1) ;
    rect(bx+shift-1, by-1, SizeX+1, SizeY+1) ;
    rect(bx-1, by+shift-1, SizeX+1, SizeY+1) ;
    rect(bx+shift-1, by+shift-1, SizeX+1, SizeY+1) ;

    for(i=0;i<SizeX;i++)
      for(j=0;j<SizeY;j++)
      {
        ys[i][j]=0 ;
        ya[i][j]=0 ;
      }
    Start=false ;
  }    
  else if(key=='r')
  {
    if(!Start)
    {
      for(i=0;i<SizeX;i++)
        for(j=0;j<SizeY;j++)
        {
          ys[i][j]=(random(1)<probab)? 1:0 ; 
          ya[i][j]=(random(1)<probab)? 1:0 ;
        }
    }     
  }
  else if(key=='d')
  {
    if(!Start)
    {
      double step=(p1-p0)/(double)(sections-1.) ;
      int h=SizeY/sections ;
      double rnd=p0 ;

      for(i=0;i<SizeX;i++)
        for(rnd=p0,j=0;j<SizeY;j++)
        {
          if(random(1)<rnd) 
          {  
            ys[i][j]=1 ;
            ya[i][j]=1 ;      
          }
          else
          {
            ys[i][j]=0 ;
            ya[i][j]=0 ;      
          }
          if(((j+1)%h)==0)
            rnd+=step ;
        }
    }
  }
  else if(key=='p')
  {
    if(!Start)
    {
      for(i=0;i<points.length;i++)
      {
        ys[points[i][0]][points[i][1]]=1 ;  
        ya[points[i][0]][points[i][1]]=1 ;  
      }
    }
   }
  else if(key=='c')
  {
    if(!Start)
    {
      for(i=0;i<SizeX;i++)
        for(j=0;j<SizeY;j++)
        {
          if( (i-cx)*(i-cx)+(j-cy)*(j-cy)<=R1*R1 && (i-cx)*(i-cx)+(j-cy)*(j-cy)>=R2*R2 )
          {
            ys[i][j]=1 ;      
            ya[i][j]=1 ;      
          }
        }
    }
  }
  else if(key=='1')
  {
    if(!Start)
    {
      for(i=0;i<SizeX;i++)
        for(j=0;j<SizeY;j++)
          ya[i][j]=ys[i][j] ;      
    }
  }
  else if(key=='2')
  {
    if(!Start)
    {
      for(i=0;i<SizeX;i++)
        for(j=0;j<SizeY;j++)
          ys[i][j]=ya[i][j] ;      
    }      
  }
  else if(key=='s')
  {  
    Start=true ;
  }
}
